#Author: 	Hal Rottenberg <hal@halr9000.com>
# Url:		http://halr9000.com/article/tag/lib-authentication.ps1
# Purpose:	These functions allow one to easily save network credentials to disk in a relatively
#			secure manner.  The resulting on-disk credential file can only [1] be decrypted
#			by the same user account which performed the encryption.  For more details, see
#			the help files for ConvertFrom-SecureString and ConvertTo-SecureString as well as
#			MSDN pages about Windows Data Protection API.
#			[1]: So far as I know today.  Next week I'm sure a script kiddie will break it.
#
# Usage:	Export-PSCredential [-Credential <PSCredential object>] [-Path <file to export>]
#
#			If Credential is not specififed, user is prompted by Get-Credential cmdlet.
#			If not specififed, Path is "./credentials.enc.xml".
#			Output: FileInfo object referring to saved credentials
#
#			Import-PSCredential [-Path <file to import>]
#
#			If not specififed, Path is "./credentials.enc.xml".
#			Output: PSCredential object

function Export-PSCredential {
	param ( $Credential = (Get-Credential), $Path = "$($pshome)\credentials.enc.xml" )
	
	# Test for valid credential object
	if ( !$Credential -or ( $Credential -isnot [system.Management.Automation.PSCredential] ) ) {
		Throw "You must specify a credential object to export to disk."
	}
	
	# Create temporary object to be serialized to disk
	$export = "" | Select-Object Username, EncryptedPassword
	
	# Give object a type name which can be identified later
	$export.PSObject.TypeNames.Insert(0,ExportedPSCredential)
	
	$export.Username = $Credential.Username

	# Encrypt SecureString password using Data Protection API
	# Only the current user account can decrypt this cipher
	$export.EncryptedPassword = $Credential.Password | ConvertFrom-SecureString

	# Export using the Export-Clixml cmdlet
	$export | Export-Clixml $Path
	Write-Host -foregroundcolor Green "Credentials saved to: " -noNewLine

	# Return FileInfo object referring to saved credentials
	Get-Item $Path
}

function Import-PSCredential {
	param ( $Path = "$($pshome)\credentials.enc.xml" )

	# Import credential file
	$import = Import-Clixml $Path 
	
	# Test for valid import
	if ( $import.PSObject.TypeNames -notcontains 'Deserialized.ExportedPSCredential' ) {
		Throw "Input is not a valid ExportedPSCredential object, exiting."
	}
	$Username = $import.Username
	
	# Decrypt the password and store as a SecureString object for safekeeping
	$SecurePass = $import.EncryptedPassword | ConvertTo-SecureString
	
	# Build the new credential object
	$Credential = New-Object System.Management.Automation.PSCredential $Username, $SecurePass
	Write-Output $Credential
}
#######################
<#
.SYNOPSIS
Writes data only to SQL Server tables.
.DESCRIPTION
Writes data only to SQL Server tables. However, the data source is not limited to SQL Server; any data source can be used, as long as the data can be loaded to a DataTable instance or read with a IDataReader instance.
.INPUTS
None
    You cannot pipe objects to Write-DataTable
.OUTPUTS
None
    Produces no output
.EXAMPLE
$dt = Invoke-Sqlcmd2 -ServerInstance "Z003\R2" -Database pubs "select *  from authors"
Write-DataTable -ServerInstance "Z003\R2" -Database pubscopy -TableName authors -Data $dt
This example loads a variable dt of type DataTable from query and write the datatable to another database
.NOTES
Write-DataTable uses the SqlBulkCopy class see links for additional information on this class.
Version History
v1.0   - Chad Miller - Initial release
v1.1   - Chad Miller - Fixed error message
.LINK
http://msdn.microsoft.com/en-us/library/30c3y597%28v=VS.90%29.aspx
#>
function Write-DataTable
{
    [CmdletBinding()]
    param(
    [Parameter(Position=0, Mandatory=$true)] [string]$ServerInstance,
    [Parameter(Position=1, Mandatory=$true)] [string]$Database,
    [Parameter(Position=2, Mandatory=$true)] [string]$TableName,
    [Parameter(Position=3, Mandatory=$true,ValueFromPipeline = $true,ValueFromPipelinebyPropertyname = $true) ] $Data,
    [Parameter(Position=4, Mandatory=$false)] [string]$Username,
    [Parameter(Position=5, Mandatory=$false)] [string]$Password,
    [Parameter(Position=6, Mandatory=$false)] [Int32]$BatchSize=50000,
    [Parameter(Position=7, Mandatory=$false)] [Int32]$QueryTimeout=0,
    [Parameter(Position=8, Mandatory=$false)] [Int32]$ConnectionTimeout=15
    )
    
    $conn=new-object System.Data.SqlClient.SQLConnection

    if ($Username)
    { $ConnectionString = "Server={0};Database={1};User ID={2};Password={3};Trusted_Connection=False;Connect Timeout={4}" -f $ServerInstance,$Database,$Username,$Password,$ConnectionTimeout }
    else
    { $ConnectionString = "Server={0};Database={1};Integrated Security=True;Connect Timeout={2}" -f $ServerInstance,$Database,$ConnectionTimeout }

    $conn.ConnectionString=$ConnectionString

    try
    {
        $conn.Open()
        $bulkCopy = new-object ("Data.SqlClient.SqlBulkCopy") $connectionString
        $bulkCopy.DestinationTableName = $tableName
        $bulkCopy.BatchSize = $BatchSize
        $bulkCopy.BulkCopyTimeout = $QueryTimeOut
        $bulkCopy.WriteToServer($Data)
        $conn.Close()
    }
    catch
    {
        $ex = $_.Exception
        Write-Error "$ex.Message"
        continue
    }

} #Write-DataTable
#######################
<#
.SYNOPSIS
Runs a T-SQL script.
.DESCRIPTION
Runs a T-SQL script. Invoke-Sqlcmd2 only returns message output, such as the output of PRINT statements when -verbose parameter is specified
.INPUTS
None
    You cannot pipe objects to Invoke-Sqlcmd2
.OUTPUTS
   System.Data.DataTable
.EXAMPLE
Invoke-Sqlcmd2 -ServerInstance "MyComputer\MyInstance" -Query "SELECT login_time AS 'StartTime' FROM sysprocesses WHERE spid = 1"
This example connects to a named instance of the Database Engine on a computer and runs a basic T-SQL query.
StartTime
-----------
2010-08-12 21:21:03.593
.EXAMPLE
Invoke-Sqlcmd2 -ServerInstance "MyComputer\MyInstance" -InputFile "C:\MyFolder\tsqlscript.sql" | Out-File -filePath "C:\MyFolder\tsqlscript.rpt"
This example reads a file containing T-SQL statements, runs the file, and writes the output to another file.
.EXAMPLE
Invoke-Sqlcmd2  -ServerInstance "MyComputer\MyInstance" -Query "PRINT 'hello world'" -Verbose
This example uses the PowerShell -Verbose parameter to return the message output of the PRINT command.
VERBOSE: hello world
.NOTES
Version History
v1.0   - Chad Miller - Initial release
v1.1   - Chad Miller - Fixed Issue with connection closing
v1.2   - Chad Miller - Added inputfile, SQL auth support, connectiontimeout and output message handling. Updated help documentation
v1.3   - Chad Miller - Added As parameter to control DataSet, DataTable or array of DataRow Output type
#>
function Invoke-Sqlcmd2
{
    [CmdletBinding()]
    param(
    [Parameter(Position=0, Mandatory=$true)] [string]$ServerInstance,
    [Parameter(Position=1, Mandatory=$false)] [string]$Database,
    [Parameter(Position=2, Mandatory=$false)] [string]$Query,
    [Parameter(Position=3, Mandatory=$false)] [string]$Username,
    [Parameter(Position=4, Mandatory=$false)] [string]$Password,
    [Parameter(Position=5, Mandatory=$false)] [Int32]$QueryTimeout=600,
    [Parameter(Position=6, Mandatory=$false)] [Int32]$ConnectionTimeout=15,
    [Parameter(Position=7, Mandatory=$false)] [ValidateScript({test-path $_})] [string]$InputFile,
    [Parameter(Position=8, Mandatory=$false)] [ValidateSet("DataSet", "DataTable", "DataRow")] [string]$As="DataRow"
    )

    if ($InputFile)
    {
        $filePath = $(resolve-path $InputFile).path
        $Query =  [System.IO.File]::ReadAllText("$filePath")
    }

    $conn=new-object System.Data.SqlClient.SQLConnection
     
    if ($Username)
    { $ConnectionString = "Server={0};Database={1};User ID={2};Password={3};Trusted_Connection=False;Connect Timeout={4}" -f $ServerInstance,$Database,$Username,$Password,$ConnectionTimeout }
    else
    { $ConnectionString = "Server={0};Database={1};Integrated Security=True;Connect Timeout={2}" -f $ServerInstance,$Database,$ConnectionTimeout }

    $conn.ConnectionString=$ConnectionString
    
    #Following EventHandler is used for PRINT and RAISERROR T-SQL statements. Executed when -Verbose parameter specified by caller
    if ($PSBoundParameters.Verbose)
    {
        $conn.FireInfoMessageEventOnUserErrors=$true
        $handler = [System.Data.SqlClient.SqlInfoMessageEventHandler] {Write-Verbose "$($_)"}
        $conn.add_InfoMessage($handler)
    }
    
    $conn.Open()
    $cmd=new-object system.Data.SqlClient.SqlCommand($Query,$conn)
    $cmd.CommandTimeout=$QueryTimeout
    $ds=New-Object system.Data.DataSet
    $da=New-Object system.Data.SqlClient.SqlDataAdapter($cmd)
    [void]$da.fill($ds)
    $conn.Close()
    switch ($As)
    {
        'DataSet'   { Write-Output ($ds) }
        'DataTable' { Write-Output ($ds.Tables) }
        'DataRow'   { Write-Output ($ds.Tables[0]) }
    }

} #Invoke-Sqlcmd2
#######################
#######################
<#
.SYNOPSISfget
Creates a DataTable for an object
.DESCRIPTION
Creates a DataTable based on an objects properties.
.INPUTS
Object
    Any object can be piped to Out-DataTable
.OUTPUTS
   System.Data.DataTable
.EXAMPLE
$dt = Get-psdrive| Out-DataTable
This example creates a DataTable from the properties of Get-psdrive and assigns output to $dt variable
.NOTES
Adapted from script by Marc van Orsouw see link
Version History
v1.0  - Chad Miller - Initial Release
v1.1  - Chad Miller - Fixed Issue with Properties
v1.2  - Chad Miller - Added setting column datatype by property as suggested by emp0
v1.3  - Chad Miller - Corrected issue with setting datatype on empty properties
v1.4  - Chad Miller - Corrected issue with DBNull
v1.5  - Chad Miller - Updated example
.LINK
http://thepowershellguy.com/blogs/posh/archive/2007/01/21/powershell-gui-scripblock-monitor-script.aspx
#>
function Out-DataTable
{
    [CmdletBinding()]
    param([Parameter(Position=0, Mandatory=$true, ValueFromPipeline = $true)] [PSObject[]]$InputObject)

    Begin
    {
        $dt = new-object Data.datatable  
        $First = $true 
    }
    Process
    {
        foreach ($object in $InputObject)
        {
            $DR = $DT.NewRow()  
            foreach($property in $object.PsObject.get_properties())
            {  
                if ($first)
                {  
                    $Col =  new-object Data.DataColumn  
                    $Col.ColumnName = $property.Name.ToString()  
                    if ($property.value)
                    {
                        if ($property.value -isnot [System.DBNull])
                        { $Col.DataType = $property.value.gettype() }
                    }
                    $DT.Columns.Add($Col)
                }  
                if ($property.IsArray)
                { $DR.Item($property.Name) =$property.value | ConvertTo-XML -AS String -NoTypeInformation -Depth 1 }  
                else { $DR.Item($property.Name) = $property.value }  
            }  
            $DT.Rows.Add($DR)  
            $First = $false
        }
    } 
     
    End
    {
        Write-Output @(,($dt))
    }

} #Out-DataTable

function Get-Updates { 

[CmdletBinding()]

	Param (
		[Parameter(position=0,Mandatory = $true,ValueFromPipeline = $true,ValueFromPipelinebyPropertyName=$true)][String] $ComputerName

	)
	begin 
	{
		[System.Reflection.Assembly]::LoadWithPartialName('Microsoft.Update.Session') | Out-Null
	}
	process {
	
		$session =  [activator]::CreateInstance([type]::GetTypeFromProgID("Microsoft.Update.Session",$ComputerName))
		$us = $session.CreateUpdateSearcher()
		$qtd = $us.GetTotalHistoryCount()
		$hot = $us.QueryHistory(1, $qtd)
	
		$hot 
	}

}


function Get-ProcessBlockedInfo { param ($myevent,$InstanceName)
	
	#Find The blocked spid
	$SPIDBlocked=""
	$matches.Clear | Out-Null
	$found = $myEvent.textdata.substring(1,400)  -match '^.*spid="(\d+)".*$' 
	if ($found)
	{
		
		
		$SPIDBlocked = $matches[1] 
		
		$Data = Invoke-Sqlcmd -ServerInstance "$($InstanceName)" -Database Master -Query "Sp_whoisactive $SPIDBlocked, @find_block_leaders =1,@get_plans =2,@get_full_inner_text=1" | `
				select 	"dd hh:mm:ss.mss",
						session_id,	sql_text,query_plan,	login_name,	wait_info,	CPU,
						@{Expression={($myEvent.Duration / 1000)};Label="Duration"},
						tempdb_allocations,
						tempdb_current,	blocking_session_id,	blocked_session_count,	reads,	writes,	physical_reads,	
						used_memory	,status,	open_tran_count,	host_name,	database_name,	program_name,	
						start_time,	login_time,	collection_time

		
		} 
}

function Show-BalloonTip {
 # Requires -Version 2.0
 [CmdletBinding()]
 param(
  [Parameter(Mandatory = $true, Position = 0)]
  [ValidateNotNull()]
  [String]
  $BalloonTipText,
  [Parameter(Position = 1)]
  [String]
  $BalloonTipTitle = 'PowerShell Event Notificaton',
  [Parameter(Position = 2)]
  [ValidateSet('Error', 'Info', 'None', 'Warning')]
  [String]
  $BalloonTipIcon = 'Info'
 )
 end {
  Add-Type -AssemblyName System.Windows.Forms
  Add-Type -AssemblyName System.Drawing
  [Windows.Forms.ToolTipIcon]$BalloonTipIcon = $BalloonTipIcon
  $NotifyIcon = New-Object Windows.Forms.NotifyIcon -Property @{
   BalloonTipIcon = $BalloonTipIcon
   BalloonTipText = $BalloonTipText
   BalloonTipTitle = $BalloonTipTitle
   Icon = [Drawing.Icon]::ExtractAssociatedIcon((Get-Command powershell).Path)
   Text = -join $BalloonTipText[0..62]
   Visible = $true
  }
  switch ($BalloonTipIcon) {
   Error {[Media.SystemSounds]::Hand.Play()}
   Info {[Media.SystemSounds]::Asterisk.Play()}
   None {[Media.SystemSounds]::Beep.Play()}
   Warning {[Media.SystemSounds]::Exclamation.Play()}
  }
  $NotifyIcon.ShowBalloonTip(0)
  switch ([Threading.Thread]::CurrentThread.ApartmentState) {
   STA {
    $null = Register-ObjectEvent -InputObject $NotifyIcon -EventName BalloonTipClosed -Action {
     $Sender.Dispose()
     Unregister-Event $EventSubscriber.SourceIdentifier
     Remove-Job $EventSubscriber.Action
    }
   }
   MTA {
    continue
   }
  }
 }
 
 <#
 .Synopsis
     Show-BalloonTip displays a balloon tip message in the taskbar's notification area.
 .Description
     Show-BalloonTip displays a balloon tip message in the taskbar's notification area.
 .Parameter BalloonTipText
     Sets the text to display in the balloon tip. This argument is required.
 .Parameter BalloonTipTitle
     Sets the title of the balloon tip. The default value is 'PowerShell Event Notification'.
 .Parameter BalloonTipIcon
     Sets the icon of the balloon tip. Valid values are Error, Info, None or Warning. The default value is 'Info'.
 .Example
     Show-BalloonTip 'PowerShell v2 will blow your mind away!' 'Did you know?' Warning
     Description
     ===========
     Displays a balloon tip warning in the taskbar's notification area.
 .Example
     Show-BalloonTip 'Report is finished!'
     Description
     ===========
     Displays a balloon tip message in the taskbar's notification area usig the default title and icon.
 .Inputs
     None
 .Outputs
     None
 .Notes
     Name:      Show-BalloonTip
     Author:    Robert Robelo
     LastEdit:  03/18/2010 23:13
 #>
}

function Get-SQLObjectDescription { 
	Param ( $object)
	
	switch ($object) {
		
		8259  {"Check Constraint"}
		8260  {"Default (constraint or standalone)"} 
		8262  {"Foreign-key Constraint"} 
		8272 {"Stored Procedure"} 
		8274 {"Rule"} 
		8275 {"System Table"}
		8276 {"Trigger on Server"}
		8277 {"(User-defined) Table"}
		8278 {"View"} 
		8280 {"Extended Stored Procedure"} 
		16724 {"CLR Trigger"} 
		16964 {"Database"} 
		16975 {"Object"} 
		17222 {"FullText Catalog"} 
		17232 {"CLR Stored Procedure"} 
		17235 {"Schema"} 
		17475 {"Credential"} 
		17491 {"DDL Event"} 
		17741 {"Management Event"} 
		17747 {"Security Event"} 
		17749 {"User Event"} 
		17985 {"CLR Aggregate Function"}
		17993 {"Inline Table-valued SQL Function"} 
		18000 {"Partition Function"} 
		18002 {"Replication Filter Procedure "}
		18004 {"Table-valued SQL Function"} 
		18259 {"Server Role "}
		18263 {"Microsoft Windows Group "}
		19265 {"Asymmetric Key"} 
		19277 {"Master Key "}
		19280 {"Primary Key "}
		19283 {"ObfusKey "}
		19521 {"Asymmetric Key Login "}
		19523 {"Certificate Login "}
		19538 {"Role "}
		19539 {"SQL Login "}
		19543 {"Windows Login "}
		20034 {"Remote Service Binding "}
		20036 {"Event Notification on Database"}
		20037 {"Event Notification "}
		20038 {"Scalar SQL Function "}
		20047 {"Event Notification on Object"}
		20051 {"Synonym "}
		20549 {"End Point "}
		20801 {"Adhoc Queries which may be cached"}
		20816 {"Prepared Queries which may be cached"}
		20819 {"Service Broker Service Queue "}
		20821 {"Unique Constraint "}
		21057 {"Application Role "}
		21059 {"Certificate "}
		21075 {"Server "}
		21076 {"Transact-SQL Trigger "}
		21313 {"Assembly "}
		21318 {"CLR Scalar Function "}
		21321 {"Inline scalar SQL Function "}
		21328 {"Partition Scheme "}
		21333 {"User "}
		21571 {"Service Broker Service Contract "}
		21572 {"Trigger on Database"}
		21574 {"CLR Table-valued Function "}
		21577 {"Internal Table (For example, XML Node Table, Queue Table.) "}
		21581 {"Service Broker Message Type"} 
		21586 {"Service Broker Route "}
		21587 {"Statistics "}
		22099 {"Service Broker Service "}
		22601 {"Index "}
		22604 {"Certificate Login "}
		22611 {"XMLSchema"}
		22868 {"Type"}
	}
}

function Get-LoadTable {
		param ($myevent)
		
		$data = $myevent | select	ApplicationName,
									DatabaseID,
									DatabaseName,
									DBUserName,
									HostName,
									LoginName,
									NTDomainName,
									NTUserName,
									ObjectName,
									ObjectType,
									@{Expression={Get-SQLObjectDescription ObjectType };Label="ObjectTypeDescription"},
									Permissions,
									@{Expression={	Switch(Permissions) { 
														1 {"Select ALL"}
														2 {"Update ALL"}
														4 {"References ALL"}
														8 {"Insert"}
														16 {"Delete"}
														32 {"Execute (procedures only)"}
													} };Label="PermissionsDescription"},
									ServerName,
									SessionLoginName,
									SPID,
									StartTime,
									Success
									TextData
		
		$valuedatatable = Out-DataTable -InputObject $data
		Write-DataTable -ServerInstance Vader -Database Test -TableName LoadTable -data $valuedatatable
}
		
function get-myfunction  {
		param ($myevent)
		$myevent | select nome
		
}

Function Get-DiskSpace {
 
 	[CmdletBinding()]
     param(
 			[Parameter(Position=0, Mandatory=$true,ValueFromPipeline = $true,ValueFromPipelinebyPropertyname = $true)] [string[]] $Servername,
 			[Parameter(Position=1, Mandatory=$false)] [string]  $unit= "GB"
 	)
 
 	process {
		foreach ($Server in $Servername) {
	
		    $measure = "1$unit"
		    Get-WmiObject -computername $Server -query "select SystemName, Name, DriveType, FileSystem, FreeSpace, Capacity, Label
		    												from Win32_Volume where DriveType = 2 or DriveType = 3" `
		    	| select 	SystemName `
					      , Name `
					      , @{Label="SizeIn$unit";Expression={"{0:n2}" -f($_.Capacity/$measure)}} `
					      , @{Label="FreeIn$unit";Expression={"{0:n2}" -f($_.freespace/$measure)}} `
					      , @{Label="PercentFree";Expression={"{0:n2}" -f(($_.freespace / $_.Capacity) * 100)}} `
					      ,  Label
		}
	}
}

#requires -version 2

function Export-Xls{
	<#
.SYNOPSIS
	Saves Microsoft .NET Framework objects to a worksheet in an XLS file
.DESCRIPTION
	The Export-Xls function allows you to save Microsoft .NET Framework objects
	to a named worksheet in an Excel file (type XLS). The position of the
	worksheet can be specified.
.NOTES
	Author:  Luc Dekens
.PARAMETER InputObject
	Specifies the objects to be written to the worksheet. The parameter accepts
	objects through the pipeline.
.PARAMETER Path
	Specifies the path to the XLS file.
.PARAMETER WorksheetName
	The name for the new worksheet. If not specified the name will
	be "Sheet" followed by the "Ticks" value
.PARAMETER SheetPosition
	Specifies where the new worksheet will be inserted in the series of
	existing worksheets. You can specify "begin" or "end". The default
	is "begin".
.PARAMETER ChartType
	Specifies the type of chart you want add to the worksheet.
	All types in the [microsoft.Office.Interop.Excel.XlChartType]
	enumeration are accepted.
.PARAMETER NoTypeInformation
	Omits the type information from the worksheet. The default is to
	include the "#TYPE" line.
.PARAMETER AppendWorksheet
	Specifies if the worksheet should keep or remove the existing
	worksheet in the spreadsheet. The default is to append.
.EXAMPLE
	PS> $data = Get-Process | Select-Object Name, Id, WS
	PS> Export-Xls $data C:\Reports\MyWkb.xls -WorksheetName "WS" -AppendWorksheet:$false
.EXAMPLE
	PS> $data = Get-Process | Select-Object Name, Id, WS
	PS> Export-Xls $data C:\Reports\MyWkb.xls -SheetPosition "end"
.EXAMPLE
	PS> $data = Get-Process | Select-Object Name, Id, WS
	PS> Export-Xls $data C:\Reports\MyWkb.xls -WorksheetName "WS" -ChartType "xlColumnClustered"
#>
	param(
	[parameter(ValueFromPipeline = $true,Position=1)]
	[ValidateNotNullOrEmpty()]
	$InputObject,
	[parameter(Position=2)]
	[ValidateNotNullOrEmpty()]
	[string]$Path,
	[string]$WorksheetName = ("Sheet " + (Get-Date).Ticks),
	[string]$SheetPosition = "begin",
	[PSObject]$ChartType,
	[switch]$NoTypeInformation = $true,
	[switch]$AppendWorksheet = $true
	)

	begin{


		[System.Reflection.Assembly]::LoadWithPartialName("Microsoft.Office.Interop.Excel")
		if($ChartType){
			[microsoft.Office.Interop.Excel.XlChartType]$ChartType = $ChartType
		}

		function Set-ClipBoard{
			param(
				[string]$text
			)
			process{
				Add-Type -AssemblyName System.Windows.Forms
				$tb = New-Object System.Windows.Forms.TextBox
				$tb.Multiline = $true
				$tb.Text = $text
				$tb.SelectAll()
				$tb.Copy()
			}
		}

		function Add-Array2Clipboard {
			param (
				[PSObject[]]$ConvertObject,
				[switch]$Header
			)
			process{
				$array = @()

				if ($Header) {
					$line =""
					$ConvertObject | Get-Member -MemberType Property,NoteProperty,CodeProperty | Select -Property Name | %{
						$line += ($_.Name.tostring() + "`t")
					}
					$array += ($line.TrimEnd("`t") + "`r")
				}
				else {
					foreach($row in $ConvertObject){
						$line =""
						$row | Get-Member -MemberType Property,NoteProperty | %{
							$Name = $_.Name
							if(!$Row.$Name -and $row.$name -ne 0 ){$Row.$Name = ""}
							$line += ([string]$Row.$Name + "`t")
						}
						$array += ($line.TrimEnd("`t") + "`r")
					}
				}
				Set-ClipBoard $array
			}
		}

		$excelApp = New-Object -ComObject "Excel.Application"
		$originalAlerts = $excelApp.DisplayAlerts
		$excelApp.DisplayAlerts = $false
		if(Test-Path -Path $Path -PathType "Leaf"){
			$workBook = $excelApp.Workbooks.Open($Path)
		}
		else{
			$workBook = $excelApp.Workbooks.Add()
		}
		$sheet = $excelApp.Worksheets.Add($workBook.Worksheets.Item(1))
		if(!$AppendWorksheet){
			$workBook.Sheets | where {$_ -ne $sheet} | %{$_.Delete()}
		}
		$sheet.Name = $WorksheetName
		if($SheetPosition -eq "end"){
			$nrSheets = $workBook.Sheets.Count
			2..($nrSheets) |%{
				$workbook.Sheets.Item($_).Move($workbook.Sheets.Item($_ - 1))
			}
		}
		$sheet.Activate()
		$array = @()
	}

	process{
		$array += $InputObject
	}

	end{
		Add-Array2Clipboard $array -Header:$True
		$selection = $sheet.Range("A1")
		$selection.Select() | Out-Null
		$sheet.Paste()
		$Sheet.UsedRange.HorizontalAlignment = [microsoft.Office.Interop.Excel.XlHAlign]::xlHAlignCenter
		Add-Array2Clipboard $array
		$selection = $sheet.Range("A2")
		$selection.Select() | Out-Null
		$sheet.Paste() | Out-Null
		$selection = $sheet.Range("A1")
		$selection.Select() | Out-Null

		$sheet.UsedRange.EntireColumn.AutoFit() | Out-Null
		$workbook.Sheets.Item(1).Select()
		if($ChartType){
			$sheet.Shapes.AddChart($ChartType) | Out-Null
		}
		$xlFixedFormat = [Microsoft.Office.Interop.Excel.XlFileFormat]::xlWorkbookDefault 
		
		
		[threading.thread]::CurrentThread.CurrentCulture = 'en-US'
		$workbook.SaveAs($path)
		$excelApp.DisplayAlerts = $originalAlerts
		$excelApp.Quit()
		Stop-Process -Name "Excel"
	}
}


function New-Zip {

	param(
		[Parameter(Mandatory=$true,
		Position=0,
		ValueFromPipeline=$true)]
		[String]
		$Path,

		[Switch]
		$PassThru,

		[Switch]
		$Force
	)
	
	process 	{

   		if (Test-Path $path) {
    		if (-not $Force) { 
     			return
    		}
   		}
   		Set-Content $path ("PK" + [char]5 + [char]6 + ("$([char]0)" * 18))
   		$item = Get-Item $path
   		$item.IsReadOnly = $false 
   		if ($passThru) { $item } 
  
 	}
}
 


Function Copy-ToZip
{

	param(
			[Parameter(Mandatory=$true,
			Position=0,
			ValueFromPipelineByPropertyName=$true)]
			[Alias('FullName')]
			[String]$File,

			[Parameter(Mandatory=$true,Position=1)]
			[String]$ZipFile,

			[Switch]$HideProgress,

			[Switch]$Force
	)
	begin 	{

		$ShellApplication = New-Object -ComObject Shell.Application
		if (-not (Test-Path $ZipFile)) {
		New-Zip $ZipFile
		}
		$Path = Resolve-Path $ZipFile
		$ZipPackage =$ShellApplication.Namespace("$Path")

	}
	process {

		$RealFile = Get-Item $File
		if (-not $RealFile) { return }    
		if (-not $hideProgress) {
			$perc +=5 
			if ($perc -gt 100) { $perc = 0 } 
			Write-Progress "Copying to $ZipFile" $RealFile.FullName -PercentComplete $perc
		}
		$Flags = 0
		if ($force) {
			$flags = 16 -bor 1024 -bor 64 -bor 512
		} 
		Write-Verbose $realFile.Fullname
		$ZipPackage.CopyHere($realFile.Fullname, $flags)
		Start-Sleep -Milliseconds 500
    
  }
}


Function Convert-CSVToExcel
{
<#   
  .SYNOPSIS  
    Converts one or more CSV files into an excel file.
     
  .DESCRIPTION  
    Converts one or more CSV files into an excel file. Each CSV file is imported into its own worksheet with the name of the
    file being the name of the worksheet.
       
  .PARAMETER inputfile
    Name of the CSV file being converted
  
  .PARAMETER output
    Name of the converted excel file
       
  .EXAMPLE  
  Get-ChildItem *.csv | ConvertCSV-ToExcel -output 'report.xlsx'
  
  .EXAMPLE  
  ConvertCSV-ToExcel -inputfile 'file.csv' -output 'report.xlsx'
    
  .EXAMPLE      
  ConvertCSV-ToExcel -inputfile @("test1.csv","test2.csv") -output 'report.xlsx'
  
  .NOTES
  Author: Boe Prox									      
  Date Created: 01SEPT210								      
  Last Modified:  
     
#>
     
#Requires -version 2.0  
[CmdletBinding(
    SupportsShouldProcess = $True,
    ConfirmImpact = 'low',
	DefaultParameterSetName = 'file'
    )]
Param (    
    [Parameter(
     ValueFromPipeline=$True,
	 ValueFromPipelineByPropertyName=$True,
     Position=0,
     Mandatory=$True,
     HelpMessage="Name of CSV/s to import")]
     [ValidateNotNullOrEmpty()]
    [string[]]$inputfile,
    [Parameter(
     ValueFromPipeline=$False,
     Position=1,
     Mandatory=$True,
     HelpMessage="Name of excel file output")]
     [ValidateNotNullOrEmpty()]
    [string]$output    
    )

Begin {     

		Function Release-Ref ($ref) 
		    {
		        ([System.Runtime.InteropServices.Marshal]::ReleaseComObject(
		        [System.__ComObject]$ref) -gt 0)
		        [System.GC]::Collect()
		        [System.GC]::WaitForPendingFinalizers() 
		    }

    #Configure regular expression to match full path of each file
	$COunter = 1
	$SName1 = @()
    [regex]$regex = "^\w\:\\"
    
    #Find the number of CSVs being imported
    $count = ($inputfile.count -1)
   
    #Create Excel Com Object
    $excel = new-object -com excel.application
    
    #Disable alerts
    $excel.DisplayAlerts = $False

    #Show Excel application
    $excel.Visible = $False

    #Add workbook
    $workbook = $excel.workbooks.Add()

    #Remove other worksheets
    #$workbook.worksheets.Item(2).delete()
    #After the first worksheet is removed,the next one takes its place
    #$workbook.worksheets.Item(2).delete()   

    #Define initial worksheet number
    $i = 1
    }

Process {
    ForEach ($input in $inputfile) {
        #If more than one file, create another worksheet for each file
        If ($i -gt 1) {
            $workbook.worksheets.Add() | Out-Null
            }
        #Use the first worksheet in the workbook (also the newest created worksheet is always 1)
        $worksheet = $workbook.worksheets.Item(1)
        #Add name of CSV as worksheet name
		Write-Host "Processing $((GCI $input).basename)"
		$Sname = "$((GCI $input).basename)                                          "

		$Sname = $Sname.substring(0,15)
		if ($Sname -contains $SName1) {
			$Sname	= "$($Sname.substring(0,13))_$($Counter)"
			$COunter++
		}	
		$SName1	+=$Sname

		

        $worksheet.name = $Sname

        #Open the CSV file in Excel, must be converted into complete path if no already done
        If ($regex.ismatch($input)) {
            $tempcsv = $excel.Workbooks.Open($input) 
            }
        ElseIf ($regex.ismatch("$($input.fullname)")) {
            $tempcsv = $excel.Workbooks.Open("$($input.fullname)") 
            }    
        Else {    
            $tempcsv = $excel.Workbooks.Open("$($pwd)\$input")      
            }
        $tempsheet = $tempcsv.Worksheets.Item(1)
        #Copy contents of the CSV file
        $tempSheet.UsedRange.Copy() | Out-Null
        #Paste contents of CSV into existing workbook
        $worksheet.Paste()

        #Close temp workbook
        $tempcsv.close()

        #Select all used cells
        $range = $worksheet.UsedRange

        #Autofit the columns
        $range.EntireColumn.Autofit() | out-null
        $i++
        } 
    }        

End {
    #Save spreadsheet
    $workbook.saveas("$output")

    Write-Host -Fore Green "File saved to $pwd\$output"

    #Close Excel
    $excel.quit()  

    #Release processes for Excel
    $a = Release-Ref($range)
    }
}
function Get-WindowsUpdates { 

[CmdletBinding()]

    Param (
        [Parameter(position=0,Mandatory = $true,ValueFromPipeline =$true,ValueFromPipelinebyPropertyName=$true)][String[]]$ComputerName
    )
    begin 
    {
        [System.Reflection.Assembly]::LoadWithPartialName('Microsoft.Update.Session') | Out-Null
    }
    process {
    
		foreach ($Computer in $ComputerName) {
	
	        $session =  [activator]::CreateInstance([type]::GetTypeFromProgID("Microsoft.Update.Session", $Computer))
	        $us = $session.CreateUpdateSearcher()
	        $qtd = $us.GetTotalHistoryCount()
	        $hot = $us.QueryHistory(1, $qtd)
	    
	        foreach ($Upd in $hot) {
	            $Property = @{ 
							'CurrentDate'=(Get-Date);
	                        'ServerName'=$Computer; 
	                        'UpdateDate'=$Upd.date ;
	                        'UpdateTitle'=$Upd.title;
	                        'UpdateDescription'=$Upd.Description;
	                        'UpdateClientApplicationID'=$Upd.ClientApplicationID
	                        }
	            Write-Output (New-Object -Type PSObject -Prop $Property)   
			}
        }
    }
}

Function Get-SqlServerInfo {

    [CmdletBinding()]
	
	
    param(
    	[Parameter(Position=0, Mandatory=$true, ValueFromPipeline = $true)] [string[]]$SQLServer
    )
	begin {
		[reflection.assembly]::LoadWithPartialName("Microsoft.SqlServer.Smo") > $null 
	}
	process {
		foreach ($SQL in $SQLServer) {
			$server = new-object ('Microsoft.SqlServer.Management.Smo.Server') $SQL
			write-output $server
		}
	}
}

#requires -version 2.0
<#
################################################################################
## Run commands in multiple concurrent pipelines
##   by Arnoud Jansveld - www.jansveld.net/powershell
##
## Basic "drop in" usage examples:
##   - Functions that accept pipelined input:
##       Without Split-Job:
##          Get-Content hosts.txt | MyFunction | Export-Csv results.csv
##       With Split-Job:
##          Get-Content hosts.txt | Split-Job {MyFunction} | Export-Csv results.csv
##   - Functions that do not accept pipelined input (use foreach):
##       Without Split-Job:
##          Get-Content hosts.txt |% { .\MyScript.ps1 -ComputerName $_ } | Export-Csv results.csv
##       With Split-Job:
##          Get-Content hosts.txt | Split-Job {%{ .\MyScript.ps1 -ComputerName $_ }} | Export-Csv results.csv
##
## Example with an imported function:
##       function Test-WebServer ($ComputerName) {
##           $WebRequest = [System.Net.WebRequest]::Create("http://$ComputerName")
##           $WebRequest.GetResponse()
##       }
##       Get-Content hosts.txt | Split-Job {%{Test-WebServer $_ }} -Function Test-WebServer | Export-Csv results.csv
##
## Example with importing a module
##       Get-Content Clusters.txt | Split-Job { % { Get-Cluster -Name $_ } } -InitializeScript { Import-Module FailoverClusters }
##	
##
## Version History
## 1.2    Changes by Stephen Mills - stephenmills at hotmail dot com
##        Only works with PowerShell V2
##        Modified error output to use ErrorRecord parameter of Write-Error - catches Category Info then.
##        Works correctly in powershell_ise.  Previous version would let pipelines continue if ESC was pressed.  If Escape pressed, then it will do an async cancel of the pipelines and exit.
##        Add seconds remaining to progress bar
##        Parameters Added and related functionality:
##           InitializeScript - allows to have custom scripts to initilize ( Import-Module ...), parameter might be renamed Begin in the future.
##           MaxDuration - Cancel all pending and in process items in queue if the number of seconds is reached before all input is done.
##           ProgressInfo - Allows you to add additional text to progress bar
##           NoProgress - Hide Progress Bar
##           DisplayInterval - frequency to update Progress bar in milliseconds
##           InputObject - not yet used, planned to be used in future to support start processing the queue before pipeline isn't finished yet
##        Added example for importing a module.
## 1.0    First version posted on poshcode.org
##        Additional runspace error checking and cleanup
## 0.93   Improve error handling: errors originating in the Scriptblock now
##        have more meaningful output
##        Show additional info in the progress bar (thanks Stephen Mills)
##        Add SnapIn parameter: imports (registered) PowerShell snapins
##        Add Function parameter: imports functions
##        Add SplitJobRunSpace variable; allows scripts to test if they are
##        running in a runspace
## 0.92   Add UseProfile switch: imports the PS profile
##        Add Variable parameter: imports variables
##        Add Alias parameter: imports aliases
##        Restart pipeline if it stops due to an error
##        Set the current path in each runspace to that of the calling process
## 0.91   Revert to v 0.8 input syntax for the script block
##        Add error handling for empty input queue
## 0.9    Add logic to distinguish between scriptblocks and cmdlets or scripts:
##        if a ScriptBlock is specified, a foreach {} wrapper is added
## 0.8    Adds a progress bar
## 0.7    Stop adding runspaces if the queue is already empty
## 0.6    First version. Inspired by Gaurhoth's New-TaskPool script
################################################################################
#>

function Split-Job
{
	param (
		[Parameter(Position=0, Mandatory=$true)]$Scriptblock,
		[Parameter()][int]$MaxPipelines=10,
		[Parameter()][switch]$UseProfile,
		[Parameter()][string[]]$Variable,
		[Parameter()][string[]]$Function = @(),
		[Parameter()][string[]]$Alias = @(),
		[Parameter()][string[]]$SnapIn,
		[Parameter()][float]$MaxDuration = $( [Int]::MaxValue ),
		[Parameter()][string]$ProgressInfo ='',
		[Parameter()][int]$ProgressID = 0,
		[Parameter()][switch]$NoProgress,
		[Parameter()][int]$DisplayInterval = 300,
		[Parameter()][scriptblock]$InitializeScript,
		[Parameter(ValueFromPipeline=$true)][object[]]$InputObject
	)

	begin
	{
		$StartTime = Get-Date
		#$DisplayTime = $StartTime.AddMilliseconds( - $DisplayInterval )
		$ExitForced = $false


		 function Init ($InputQueue){
			# Create the shared thread-safe queue and fill it with the input objects
			$Queue = [Collections.Queue]::Synchronized([Collections.Queue]@($InputQueue))
			$QueueLength = $Queue.Count
			# Do not create more runspaces than input objects
			if ($MaxPipelines -gt $QueueLength) {$MaxPipelines = $QueueLength}
			# Create the script to be run by each runspace
			$Script  = "Set-Location '$PWD'; "
			$Script += {
				$SplitJobQueue = $($Input)
				& {
					trap {continue}
					while ($SplitJobQueue.Count) {$SplitJobQueue.Dequeue()}
				} }.ToString() + '|' + $Scriptblock

			# Create an array to keep track of the set of pipelines
			$Pipelines = New-Object System.Collections.ArrayList

			# Collect the functions and aliases to import
			$ImportItems = ($Function -replace '^','Function:') +
				($Alias -replace '^','Alias:') |
				Get-Item | select PSPath, Definition
			$stopwatch = [System.Diagnostics.Stopwatch]::StartNew()
		}

		function Add-Pipeline {
			# This creates a new runspace and starts an asynchronous pipeline with our script.
			# It will automatically start processing objects from the shared queue.
			$Runspace = [System.Management.Automation.Runspaces.RunspaceFactory]::CreateRunspace($Host)
			$Runspace.Open()
			if (!$?) { throw "Could not open runspace!" }
			$Runspace.SessionStateProxy.SetVariable('SplitJobRunSpace', $True)

			function CreatePipeline
			{
				param ($Data, $Scriptblock)
				$Pipeline = $Runspace.CreatePipeline($Scriptblock)
				if ($Data)
				{
					$Null = $Pipeline.Input.Write($Data, $True)
					$Pipeline.Input.Close()
				}
				$Null = $Pipeline.Invoke()
				$Pipeline.Dispose()
			}

			# Optionally import profile, variables, functions and aliases from the main runspace
			
			if ($UseProfile)
			{
				CreatePipeline -Script "`$PROFILE = '$PROFILE'; . `$PROFILE"
			}

			if ($Variable)
			{
				foreach ($var in (Get-Variable $Variable -Scope 2))
				{
					trap {continue}
					$Runspace.SessionStateProxy.SetVariable($var.Name, $var.Value)
				}
			}
			if ($ImportItems)
			{
				CreatePipeline $ImportItems {
					foreach ($item in $Input) {New-Item -Path $item.PSPath -Value $item.Definition}
				}
			}
			if ($SnapIn)
			{
				CreatePipeline (Get-PSSnapin $Snapin -Registered) {$Input | Add-PSSnapin}
			}
			
			#Custom Initialization Script for startup of Pipeline - needs to be after other other items added.
			if ($InitializeScript -ne $null)
			{
				CreatePipeline -Scriptblock $InitializeScript
			}

			$Pipeline = $Runspace.CreatePipeline($Script)
			$Null = $Pipeline.Input.Write($Queue)
			$Pipeline.Input.Close()
			$Pipeline.InvokeAsync()
			$Null = $Pipelines.Add($Pipeline)
		}

		function Remove-Pipeline ($Pipeline)
		{
			# Remove a pipeline and runspace when it is done
			$Pipeline.RunSpace.CloseAsync()
			#Removed Dispose so that Split-Job can be quickly aborted even if currently running something waiting for a timeout.
			#Added call to [System.GC]::Collect() at end of script to free up what memory it can.
			#$Pipeline.Dispose()
			$Pipelines.Remove($Pipeline)
		}
	}

	end
	{
		


		# Main
		# Initialize the queue from the pipeline
		. Init $Input
		# Start the pipelines
		try
		{
			while ($Pipelines.Count -lt $MaxPipelines -and $Queue.Count) {Add-Pipeline}

			# Loop through the pipelines and pass their output to the pipeline until they are finished
			while ($Pipelines.Count)
			{
				# Only update the progress bar once per $DisplayInterval
				if (-not $NoProgress -and $Stopwatch.ElapsedMilliseconds -ge $DisplayInterval)
				{
					$Completed = $QueueLength - $Queue.Count - $Pipelines.count
					$Stopwatch.Reset()
					$Stopwatch.Start()
					#$LastUpdate = $stopwatch.ElapsedMilliseconds
					$PercentComplete = (100 - [Int]($Queue.Count)/$QueueLength*100)
					$Duration = (Get-Date) - $StartTime
					$DurationString = [timespan]::FromSeconds( [Math]::Floor($Duration.TotalSeconds)).ToString()
					$ItemsPerSecond = $Completed / $Duration.TotalSeconds
					$SecondsRemaining = [math]::Round(($QueueLength - $Completed)/ ( .{ if ($ItemsPerSecond -eq 0 ) { 0.001 } else { $ItemsPerSecond}}))
					
					Write-Progress -Activity "** Split-Job **  *Press Esc to exit*  Next item: $(trap {continue}; if ($Queue.Count) {$Queue.Peek()})" `
						-status "Queues: $($Pipelines.Count) QueueLength: $($QueueLength) StartTime: $($StartTime)  $($ProgressInfo)" `
						-currentOperation  "$( . { if ($ExitForced) { 'Aborting Job!   ' }})Completed: $($Completed) Pending: $($QueueLength- ($QueueLength-($Queue.Count + $Pipelines.Count))) RunTime: $($DurationString) ItemsPerSecond: $([math]::round($ItemsPerSecond, 3))"`
						-PercentComplete $PercentComplete `
						-Id $ProgressID `
						-SecondsRemaining $SecondsRemaining
				}	
				foreach ($Pipeline in @($Pipelines))
				{
					if ( -not $Pipeline.Output.EndOfPipeline -or -not $Pipeline.Error.EndOfPipeline)
					{
						$Pipeline.Output.NonBlockingRead()
						$Pipeline.Error.NonBlockingRead() | % { Write-Error -ErrorRecord $_ }

					} else
					{
						# Pipeline has stopped; if there was an error show info and restart it			
						if ($Pipeline.PipelineStateInfo.State -eq 'Failed')
						{
							Write-Error $Pipeline.PipelineStateInfo.Reason
							
							# Restart the runspace
							if ($Queue.Count -lt $QueueLength) {Add-Pipeline}
						}
						Remove-Pipeline $Pipeline
					}
					if ( ((Get-Date) - $StartTime).TotalSeconds -ge $MaxDuration -and -not $ExitForced)
					{
						Write-Warning "Aborting job! The MaxDuration of $MaxDuration seconds has been reached. Inputs that have not been processed will be skipped."
						$ExitForced=$true
					}
					
					if ($ExitForced) { $Pipeline.StopAsync(); Remove-Pipeline $Pipeline }
				}
				while ($Host.UI.RawUI.KeyAvailable)
				{
					if ($Host.ui.RawUI.ReadKey('NoEcho,IncludeKeyDown,IncludeKeyUp').VirtualKeyCode -eq 27 -and !$ExitForced)
					{
						$Queue.Clear();
						Write-Warning 'Aborting job! Escape pressed! Inputs that have not been processed will be skipped.'
						$ExitForced = $true;
						#foreach ($Pipeline in @($Pipelines))
						#{
						#	$Pipeline.StopAsync()
						#}
					}		
				}
				if ($Pipelines.Count) {Start-Sleep -Milliseconds 50}
			}

			#Clear the Progress bar so other apps don't have to keep seeing it.
			Write-Progress -Completed -Activity "`0" -Status "`0"

			# Since reference to Dispose was removed.  I added this to try to help with releasing resources as possible.
			# This might be a bad idea, but I'm leaving it in for now. (Stephen Mills)
			[GC]::Collect()
		}
		finally
		{
			foreach ($Pipeline in @($Pipelines))
			{
				if ( -not $Pipeline.Output.EndOfPipeline -or -not $Pipeline.Error.EndOfPipeline)
				{
					Write-Warning 'Pipeline still runinng.  Stopping Async.'
					$Pipeline.StopAsync()
					Remove-Pipeline $Pipeline
				}
			}
		}
	}
}

Export-ModuleMember -Function *

